/*
 * This implements the gnocl::builder and gnocl::glade commands.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
    2012-12: moved into separate package
	2010-03: new command
				gnocl::builder
	2009-12: new command
				gnocl::glade
 */


#include <stdlib.h>
#include <gnocl.h>


Tcl_ObjCmdProc gnoclBuilderCmd;
Tcl_ObjCmdProc gnoclGladeCmd;

/**
\brief
\author	William J Giddings
\date	30-Oct-2010
\note
**/
int Gnoclbuilder_Init ( Tcl_Interp *interp )
{

#ifdef DEBUG_BUILDER
	printf ( "Initializing gnocl bindings to GtkSpell libraries, version %s\n", VERSION );
#endif

	if ( Tcl_InitStubs ( interp, "8.3", 0 ) == NULL )
		return TCL_ERROR;

	if ( Tcl_PkgRequire ( interp, "Gnocl", VERSION, 0 ) == NULL )
		return TCL_ERROR;

	if ( Tcl_PkgProvide ( interp, "GnoclBuilder", VERSION ) != TCL_OK )
		return TCL_ERROR;

	Tcl_CreateObjCommand ( interp, "gnocl::builder", gnoclBuilderCmd, NULL, NULL );
	Tcl_CreateObjCommand ( interp, "gnocl::glade", gnoclGladeCmd, NULL, NULL );

	return TCL_OK;
}



/**
\brief      This module implements the gnocl::builder and gnocl::glade commands
\author     William J Giddings
\date       24-11-2009
\todo       -whilst parsing the builder widget tree, assign specific handlers
            -make public other constructor functions
\since    December-09 implemented widget support
                    button, togglebutton, checkbutton, entry, spinbutton, linkbutton,
                    filebuttonchooser, colorbutton, scale button, volumebutton,
                    image, label, hscale, vscale, hscrollbar, vscrollbar, accelarator

\note      Developer Documentation
            http://library.gnome.org/devel/libglade/unstable/index.html
            http://library.gnome.org/devel/libglade/unstable/GladeXML.html
            http://library.gnome.org/devel/gtk/2.16/GtkBuilder.html

            Development

            13/03/2010  Extended buffer command to include options
                -container
                -objects

**/
#include "gnocl.h"
#include "gnoclparams.h"

#include <gdk/gdk.h>
#include <string.h>
#include <assert.h>

#include <glade/glade.h>

/* these need to be global so that they can be accessed usind other builder empowered modules */
static char *GtkTypes[] =
{
	"GtkAccelLabel",
	"GtkLabel",
	"GtkProgressBar",
	"GtkStatusbar",
	"GtkStatusIcon",
	"GtkButton",
	"GtkCheckButton",
	"GtkRadioButton",
	"GtkToggleButton",
	"GtkLinkButton",
	"GtkScaleButton",
	"GtkVolumeButton",
	"GtkEntry",
	"GtkHScale",
	"GtkVScale",
	"GtkSpinButton",
	"GtkEditable",
	"GtkTextView",
	"GtkTreeView",
	"GtkCellView",
	"GtkIconView",
	"GtkComboBox",
	"GtkComboBoxEntry",
	"GtkMenu",
	"GtkMenuBar",
	"GtkMenuItem",
	"GtkToolbar",
	"GtkColorButton",
	"GtkColorSelectionDialog",
	"GtkColorSelection",
	"GtkHSV",
	"GtkFileChooser",
	"GtkFileChooserButton",
	"GtkFileChooserDialog",
	"GtkFileChooserWidget",
	"GtkFontButton",
	"GtkFontSelectionDialog",
	"GtkAlignment",
	"GtkHBox",
	"GtkVBox",
	"GtkHButtonBox",
	"GtkVButtonBox",
	"GtkFixed",
	"GtkHPaned",
	"GtkVPaned",
	"GtkLayout",
	"GtkNotebook",
	"GtkTable",
	"GtkExpander",
	"GtkFrame",
	"GtkHSeparator",
	"GtkVSeparator",
	"GtkHScrollbar",
	"GtkVScrollbar",
	"GtkScrolledWindow",
	"GtkPrintUnixDialog",
	"GtkPageSetupUnixDialog",
	"GtkArrow",
	"GtkCalendar",
	"GtkDrawingArea",
	"GtkEventBox",
	"GtkHandleBox",
	"GtkWindow",
	"GtkToolButton",
	"GtkImage",
	"GtkRecentChooserWidget",
	"GtkAboutDialog",
	"GtkDialog",
	"GtkInputDialog",
	"GtkMessageDialog",
	"GtkRecentChooserDialog",
	"GtkAssistant",
	NULL
};

static enum  GtkTypesIdx
{
	GtkAccelLabelIdx,
	GtkLabelIdx,
	GtkProgressBarIdx,
	GtkStatusbarIdx,
	GtkStatusIconIdx,
	GtkButtonIdx,
	GtkCheckButtonIdx,
	GtkRadioButtonIdx,
	GtkToggleButtonIdx,
	GtkLinkButtonIdx,
	GtkScaleButtonIdx,
	GtkVolumeButtonIdx,
	GtkEntryIdx,
	GtkHScaleIdx,
	GtkVScaleIdx,
	GtkSpinButtonIdx,
	GtkEditableIdx,
	GtkTextViewIdx,
	GtkTreeViewIdx,
	GtkCellViewIdx,
	GtkIconViewIdx,
	GtkComboBoxIdx,
	GtkComboBoxEntryIdx,
	GtkMenuIdx,
	GtkMenuBarIdx,
	GtkMenuItemIdx,
	GtkToolbarIdx,
	GtkColorButtonIdx,
	GtkColorSelectionDialogIdx,
	GtkColorSelectionIdx,
	GtkHSVIdx,
	GtkFileChooserIdx,
	GtkFileChooserButtonIdx,
	GtkFileChooserDialogIdx,
	GtkFileChooserWidgetIdx,
	GtkFontButtonIdx,
	GtkFontSelectionDialogIdx,
	GtkAlignmentIdx,
	GtkHBoxIdx,
	GtkVBoxIdx,
	GtkHButtonBoxIdx,
	GtkVButtonBoxIdx,
	GtkFixedIdx,
	GtkHPanedIdx,
	GtkVPanedIdx,
	GtkLayoutIdx,
	GtkNotebookIdx,
	GtkTableIdx,
	GtkExpanderIdx,
	GtkFrameIdx,
	GtkHSeparatorIdx,
	GtkVSeparatorIdx,
	GtkHScrollbarIdx,
	GtkVScrollbarIdx,
	GtkScrolledWindowIdx,
	GtkPrintUnixDialogIdx,
	GtkPageSetupUnixDialogIdx,
	GtkArrowIdx,
	GtkCalendarIdx,
	GtkDrawingAreaIdx,
	GtkEventBoxIdx,
	GtkHandleBoxIdx,
	GtkWindowIdx,
	GtkToolButtonIdx,
	GtkImageIdx,
	GtkRecentChooserWidgetIdx,
	GtkAboutDialogIdx,
	GtkDialogIdx,
	GtkInputDialogIdx,
	GtkMessageDialogIdx,
	GtkRecentChooserDialogIdx,
	GtkAssistantIdx
};

/**
\brief
\author William J Giddings
\date   23/11/09
\note
*/
static int builderFunc ( ClientData data, Tcl_Interp *interp, int objc, Tcl_Obj * const objv[] )
{
#ifdef DEBUG_BUILDER
	g_print ( " builderFunc, this will provide a command to access items(?) resulting from the loading of this tree!\n" );
#endif
	return TCL_OK;
}


/**
\brief  Dummy function for assigning to newly created gnocl widget within this module
\author William J Giddings
\date   23/11/09
\note
*/
static int builderWidgetFunc ( ClientData data, Tcl_Interp *interp, int objc, Tcl_Obj * const objv[] )
{
#ifdef DEBUG_BUILDER
	g_print ( " builderWidgetFunc, this is just a stub!\n" );
#endif
	return TCL_OK;
}


/**
\brief
\author Peter G Baum
\date
\note   Borrowed from gnocl.c
**/
static void simpleDestroyFunc (
	GtkWidget *widget,
	gpointer data )
{
	const char *name = gnoclGetNameFromWidget ( widget );
	gnoclForgetWidgetFromName ( name );
	Tcl_DeleteCommand ( ( Tcl_Interp * ) data, ( char * ) name );
	g_free ( ( char * ) name );
}



/**
/brief
/author   William J Giddings
/date     24-11-2009
**/
int setGtkWidgetFunc ( Tcl_Interp *interp, GtkWidget *widget, int idx, Tcl_Obj *resList , gchar *builderName, char *name )
{
#ifdef DEBUG_BUILDER
	printf ( "setGtkWidgetFunc UI_name = %s, Gnocl_ame = %s\n", builderName, name );
#endif

	Tcl_ObjCmdProc *proc;
	int set = 0; /* flag to denote if registration functions arelady completed, ie widgets using 'para' entry /toggle */

	switch ( idx )
	{
		case GtkAssistantIdx:
			{
				proc = assistantFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkRecentChooserDialogIdx:
			{
				proc = recentChooserDialogFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkMessageDialogIdx:
			{
				DialogParams *para = g_new ( DialogParams, 1 );
				para->dialog = GTK_DIALOG ( widget );
				para->interp = interp;
				para->name = NULL;
				para->butRes = g_ptr_array_new();
				para->isModal = 1;

				proc = dialogFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkInputDialogIdx:
			{
				proc = inputDialogFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkDialogIdx:
			{
				proc = inputDialogFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkAboutDialogIdx:
			{
				proc = aboutDialogFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkLabelIdx:
			{
				LabelParams *para = g_new ( LabelParams, 1 );

				para->label = widget;
				para->interp = interp;
				para->textVariable = NULL;
				para->onChanged = NULL;
				para->inSetVar = 0;
				para->name = name;

				proc = labelFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkProgressBarIdx:
			{
				proc = progressBarFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkStatusbarIdx:
			{
				proc = statusBarFuc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkStatusIconIdx:
			{
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkButtonIdx:
			{
				ButtonParams *para = g_new ( ButtonParams, 1 );

				para->interp = interp;
				para->name = name;
				para->button = widget;
				para->alignment = gtk_alignment_new ( 0.5, 0.5, 0.0, 0.0 );;

				gtk_widget_show ( GTK_WIDGET ( para->button ) );
				/*
					char        *iconName;
					char		*data;
					char		*align;
					GtkAlignment *alignment;
				*/
				g_print ( "HERE\n" );

				proc = buttonFunc;

				Tcl_CreateObjCommand ( interp, para->name, proc, para, NULL );
				g_print ( "THERE\n" );
			}
			break;
		case GtkCheckButtonIdx:
			{

				GnoclCheckParams * para = g_new ( GnoclCheckParams, 1 );

				para->interp = interp;
				para->name = name;
				para->widget = widget;
				para->onToggled = NULL;
				para->variable = NULL;
				para->onValue = Tcl_NewIntObj ( 1 );
				Tcl_IncrRefCount ( para->onValue );
				para->offValue = Tcl_NewIntObj ( 0 );
				Tcl_IncrRefCount ( para->offValue );
				para->inSetVar = 0;


				proc = checkButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkRadioButtonIdx:
			{

				GnoclRadioParams  *para = g_new ( GnoclRadioParams, 1 );

				para->name = name;
				para->widget = widget;
				para->onToggled = NULL;
				para->onValue = NULL;
				/*
				                para->group = gnoclRadioGetGroupFromVariable (
				                                  radioButtonOptions[variableIdx].val.str );

				                if ( para->group == NULL )
				                {
				                    para->group = gnoclRadioGroupNewGroup (
				                                      radioButtonOptions[variableIdx].val.str, interp );
				                }

				                else
				                {
				                    GnoclRadioParams *p = gnoclRadioGetParam ( para->group, 0 );
				                    gtk_radio_button_set_group ( GTK_RADIO_BUTTON ( para->widget ),
				                                                 gtk_radio_button_get_group ( GTK_RADIO_BUTTON ( p->widget ) ) );
				                }

				                gnoclRadioGroupAddWidgetToGroup ( para->group, para );
				*/

				proc = radioButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkToggleButtonIdx:
			{

				GnoclToggleParams *para;

				para = g_new ( GnoclToggleParams, 1 );

				para->interp = interp;

				para->name = name;
				para->widget = widget;
				para->onToggled = NULL;
				para->variable = NULL;
				para->onValue = Tcl_NewIntObj ( 1 );
				Tcl_IncrRefCount ( para->onValue );
				para->offValue = Tcl_NewIntObj ( 0 );
				Tcl_IncrRefCount ( para->offValue );
				para->inSetVar = 0;

				proc = toggleButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkLinkButtonIdx:
			{
				proc = linkButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkScaleButtonIdx:
			{
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkVolumeButtonIdx:
			{

				proc = volumeButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkEntryIdx:
			{

				EntryParams *para = g_new ( EntryParams, 1 );
				para->entry = GTK_ENTRY ( widget );
				para->interp = interp;
				para->name = name;
				para->variable = NULL;
				para->onChanged = NULL;
				para->inSetVar = 0;
				Tcl_CreateObjCommand ( interp, name, entryFunc, para, NULL );

			}
			break;
		case GtkHScaleIdx:
		case GtkVScaleIdx:
			{
				ScaleParams    *para = g_new ( ScaleParams, 1 );

				para->scale = widget;
				para->interp = interp;
				para->variable = NULL;
				para->onValueChanged = NULL;
				para->inSetVar = 0;
				para->name = name;

				proc = scaleFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkSpinButtonIdx:
			{

				SpinButtonParams *para = g_new ( SpinButtonParams, 1 );

				para->spinButton = widget;
				para->interp = interp;
				para->variable = NULL;
				para->onValueChanged = NULL;
				para->inSetVar = 0;

				proc = spinButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkEditableIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkEditableIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkTextViewIdx:
			{
				proc = textViewFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkTreeViewIdx:
			{
				proc = treeListFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkCellViewIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkCellViewIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkIconViewIdx:
			{
				proc = iconViewFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkComboBoxIdx:
		case GtkComboBoxEntryIdx:
			{
				ComboBoxParams *para = g_new ( ComboBoxParams, 1 );
				para->interp = interp;
				para->onChanged = NULL;
				para->variable = NULL;
				para->name = name;
				para->comboBox = GTK_COMBO_BOX ( widget );
				para->inSetVar = 0;

				proc = comboBoxFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
			/*--------------- MENU HANDLERS ---------------*/
		case GtkMenuIdx:
			{
				proc = menuFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkMenuBarIdx:
			{
				proc = menuBarFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;

		case GtkMenuItemIdx:
			{

				proc = menuItemFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkAccelLabelIdx:
			{

				proc = accelaratorFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkToolbarIdx:
			{
				proc = toolBarFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkColorButtonIdx:
			{
				proc = clrButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkColorSelectionDialogIdx:
			{

				ColorSelDialogParams *para = g_new ( ColorSelDialogParams, 1 );
				para->colorSel = GTK_COLOR_SELECTION_DIALOG ( widget );

				para->interp = interp;
				para->name = name;
				para->onClicked = NULL;

				proc = colorSelDialogFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkColorSelectionIdx:
			{
				proc = colorSelectionFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkHSVIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkHSVIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkFileChooserIdx:
			{
				proc = fileChooserFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkFileChooserButtonIdx:
			{
				proc = fileChooserButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkFileChooserDialogIdx:
			{
				FileSelDialogParams *para = g_new ( FileSelDialogParams, 1 );
				para->interp = interp;
				para->getURIs = 0;
				para->fileDialog = GTK_FILE_CHOOSER_DIALOG ( widget );
				para->getURIs = 0;
				para->name = name;
				proc = fileDialogFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkFileChooserWidgetIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkFileChooserWidgetIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkFontButtonIdx:
			{
				proc = fontButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkFontSelectionDialogIdx:
			{
				FontSelDialogParams *para = NULL;
				para = g_new ( FontSelDialogParams, 1 );
				para->fontSel = GTK_FONT_SELECTION_DIALOG ( widget );
				para->interp = interp;
				para->name = name;
				para->onClicked = NULL;
				proc = fontSelDialogFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkAlignmentIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkAlignmentIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkHBoxIdx:
			{

				proc = boxFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkVBoxIdx:
			{

				proc = boxFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkHButtonBoxIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkHButtonBoxIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkVButtonBoxIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkVButtonBoxIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkFixedIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkFixedIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkHPanedIdx:
		case GtkVPanedIdx:
			{
				proc = panedFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkLayoutIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkLayoutIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkNotebookIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkNotebookIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = notebookFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkTableIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkTableIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkExpanderIdx:
			{
				proc = expanderFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkFrameIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkFrameIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkHSeparatorIdx:
		case GtkVSeparatorIdx:
			{
				proc = separatorFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkHScrollbarIdx:
		case GtkVScrollbarIdx:
			{
				ScrollParams    *para = g_new ( ScrollParams, 1 );

				para->scroll = widget;
				para->interp = interp;
				para->variable = NULL;
				para->onValueChanged = NULL;
				para->inSetVar = 0;
				para->name = name;

				proc = scrollFunc;
				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkScrolledWindowIdx:
			{
				proc =  scrlWindowFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkPrintUnixDialogIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkPrintUnixDialogIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkPageSetupUnixDialogIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( "GtkPageSetupUnixDialogIdx Not Sorted Yet!!!! %s\n", name );
#endif
				proc = builderWidgetFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkArrowIdx:
			{
				proc = arrowButtonFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkCalendarIdx:
			{
				proc = calendarFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkDrawingAreaIdx:
			{
				proc = drawingAreaFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkEventBoxIdx:
			{

				proc = eventBoxFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkHandleBoxIdx:
			{
				proc = handleBoxFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkWindowIdx:
			{

				proc = windowFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkToolButtonIdx:
			{
				/*
				extra code needed here to fully inplement gnocl widget
				see toolBar.c fuction addButton
				*/


				/**
				\brief Description yet to be added.
				\author Peter G Baum
				*/
				static const int buttonTextIdx      = 0;
				static const int buttonIconIdx      = 1;
				static const int buttonOnClickedIdx = 2;
				extern buttonOptions;

				char         *txt;
				int          isUnderline;
				GtkWidget    *icon;
				GtkToolbar *toolbar = gtk_widget_get_parent ( widget );

				if ( getTextAndIcon ( interp, toolbar, buttonOptions + buttonTextIdx,
									  buttonOptions + buttonIconIdx, &txt, &icon, &isUnderline ) != TCL_OK )
				{
					Tcl_SetResult ( interp, "GNOCL ERROR: Unable to add tool button.\n", TCL_STATIC );
					return TCL_ERROR;
				}

				ToolButtonParams *para = g_new ( ToolButtonParams, 1 );
				para->item = widget;
				para->interp = interp;
				para->name = name;
				para->onClicked = NULL;

				proc = toolButtonFunc;

				Tcl_CreateObjCommand ( interp, para->name, toolButtonFunc, para, NULL );

				Tcl_CreateObjCommand ( interp, name, proc, para, NULL );
			}
			break;
		case GtkImageIdx:
			{
				proc = imageFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}
			break;
		case GtkRecentChooserWidgetIdx:
			{
				proc = recentChooserFunc;
				Tcl_CreateObjCommand ( interp, name, proc, widget, NULL );
			}

			break;
		default:
			{
#ifdef DEBUG_BUILDER
				printf ( "do nothing\n" );
#endif
				return -1;
			}
	}

	/* perform the equivalent of gnoclRegisterWidget */
	g_signal_connect_after ( G_OBJECT ( widget ), "destroy", G_CALLBACK ( simpleDestroyFunc ), interp );
	gnoclMemNameAndWidget ( name, GTK_WIDGET ( widget ) );

	gchar str[50];
	sprintf ( str, "%s %s", name, builderName );

	Tcl_ListObjAppendElement ( interp, resList, Tcl_NewStringObj ( str, -1 ) );

	return TCL_OK;
}


/*
int gnoclRegisterWidget (
    Tcl_Interp *interp,
    GtkWidget *widget,
    Tcl_ObjCmdProc *proc )
{
    const char *name = gnoclGetAutoWidgetId();
    gnoclMemNameAndWidget ( name, widget );

    g_signal_connect_after ( G_OBJECT ( widget ), "destroy",
                             G_CALLBACK ( simpleDestroyFunc ), interp );

    if ( proc != NULL )
        Tcl_CreateObjCommand ( interp, ( char * ) name, proc, widget, NULL );

    Tcl_SetObjResult ( interp, Tcl_NewStringObj ( name, -1 ) );

    return TCL_OK;
}
*/

static void  getWidgetNames ( GSList *widgetList, GSList *widgetNames )
{

	GSList *p = NULL;
	GSList *names = NULL;
	gchar *name;

	widgetNames = NULL;

	for ( p = widgetList; p != NULL; p = p->next )
	{
		name =    ( gchar * ) gtk_buildable_get_name ( GTK_BUILDABLE ( p->data ) );
		widgetNames = g_slist_append ( widgetNames, name );
	}

}

static GSList * getBuilderObjectNames ( GtkBuilder *builder )
{

	GSList *p = NULL;
	GSList *namelist = NULL;
	const gchar *name;

	for ( p = ( GSList * ) gtk_builder_get_objects ( builder ); p != NULL; p = p->next )
	{
		name =  gtk_buildable_get_name ( GTK_BUILDABLE ( p->data ) );
		namelist = g_slist_append ( namelist, ( gchar * ) name );
	}

	return namelist;
}

static void display_list ( GSList *list )
{

	GSList *p = NULL;

	//print the list data
	for ( p = list; p != NULL; p = p->next )
	{
		printf ( "%s \n", p->data );
	}


}

// AG 28/01/13
/* Camelcased version of the get_type functions, to make our live a bit
   easier. Note that most of these types aren't supported by Gnocl right now,
   but we still list them all for future extensions. */

#define gtkAboutDialogGetType   gtk_about_dialog_get_type
#define gtkAccelGroupGetType   gtk_accel_group_get_type
#define gtkAccelLabelGetType   gtk_accel_label_get_type
#define gtkAccelMapGetType   gtk_accel_map_get_type
#define gtkAccessibleGetType   gtk_accessible_get_type
#define gtkActionGetType   gtk_action_get_type
#define gtkActionGroupGetType   gtk_action_group_get_type
#define gtkActivatableGetType   gtk_activatable_get_type
#define gtkAdjustmentGetType   gtk_adjustment_get_type
#define gtkAlignmentGetType   gtk_alignment_get_type
#define gtkAssistantGetType   gtk_assistant_get_type
#define gtkPackDirectionGetType   gtk_pack_direction_get_type
#define gtkUiManagerItemTypeGetType   gtk_ui_manager_item_type_get_type
#define gtkSpinButtonUpdatePolicyGetType   gtk_spin_button_update_policy_get_type
#define gtkNotebookTabGetType   gtk_notebook_tab_get_type
#define gtkNumberUpLayoutGetType   gtk_number_up_layout_get_type
#define gtkAnchorTypeGetType   gtk_anchor_type_get_type
#define gtkArgFlagsGetType   gtk_arg_flags_get_type
#define gtkArrowTypeGetType   gtk_arrow_type_get_type
#define gtkArrowPlacementGetType   gtk_arrow_placement_get_type
#define gtkAssistantPageTypeGetType   gtk_assistant_page_type_get_type
#define gtkAttachOptionsGetType   gtk_attach_options_get_type
#define gtkBuilderErrorGetType   gtk_builder_error_get_type
#define gtkButtonActionGetType   gtk_button_action_get_type
#define gtkButtonsTypeGetType   gtk_buttons_type_get_type
#define gtkCalendarDisplayOptionsGetType   gtk_calendar_display_options_get_type
#define gtkCellRendererModeGetType   gtk_cell_renderer_mode_get_type
#define gtkCellRendererStateGetType   gtk_cell_renderer_state_get_type
#define gtkCellRendererAccelModeGetType   gtk_cell_renderer_accel_mode_get_type
#define gtkCornerTypeGetType   gtk_corner_type_get_type
#define gtkDebugFlagGetType   gtk_debug_flag_get_type
#define gtkDeleteTypeGetType   gtk_delete_type_get_type
#define gtkDestDefaultsGetType   gtk_dest_defaults_get_type
#define gtkDragResultGetType   gtk_drag_result_get_type
#define gtkClistDragPosGetType   gtk_clist_drag_pos_get_type
#define gtkDialogFlagsGetType   gtk_dialog_flags_get_type
#define gtkDirectionTypeGetType   gtk_direction_type_get_type
#define gtkEntryIconPositionGetType   gtk_entry_icon_position_get_type
#define gtkExpanderStyleGetType   gtk_expander_style_get_type
#define gtkFileChooserActionGetType   gtk_file_chooser_action_get_type
#define gtkFileChooserConfirmationGetType   gtk_file_chooser_confirmation_get_type
#define gtkFileFilterFlagsGetType   gtk_file_filter_flags_get_type
#define gtkIconViewDropPositionGetType   gtk_icon_view_drop_position_get_type
#define gtkSensitivityTypeGetType   gtk_sensitivity_type_get_type
#define gtkWindowTypeGetType   gtk_window_type_get_type
#define gtkWrapModeGetType   gtk_wrap_mode_get_type
#define gtkWidgetFlagsGetType   gtk_widget_flags_get_type
#define gtkWidgetHelpTypeGetType   gtk_widget_help_type_get_type
#define gtkWindowPositionGetType   gtk_window_position_get_type
#define gtkTreeViewColumnSizingGetType   gtk_tree_view_column_sizing_get_type
#define gtkTreeViewDropPositionGetType   gtk_tree_view_drop_position_get_type
#define gtkButtonBoxStyleGetType   gtk_button_box_style_get_type
#define gtkCellTypeGetType   gtk_cell_type_get_type
#define gtkCtreeExpanderStyleGetType   gtk_ctree_expander_style_get_type
#define gtkCtreeExpansionTypeGetType   gtk_ctree_expansion_type_get_type
#define gtkCtreeLineStyleGetType   gtk_ctree_line_style_get_type
#define gtkCtreePosGetType   gtk_ctree_pos_get_type
#define gtkCurveTypeGetType   gtk_curve_type_get_type
#define gtkIconLookupFlagsGetType   gtk_icon_lookup_flags_get_type
#define gtkImageTypeGetType   gtk_image_type_get_type
#define gtkImPreeditStyleGetType   gtk_im_preedit_style_get_type
#define gtkImStatusStyleGetType   gtk_im_status_style_get_type
#define gtkJustificationGetType   gtk_justification_get_type
#define gtkMatchTypeGetType   gtk_match_type_get_type
#define gtkMenuDirectionTypeGetType   gtk_menu_direction_type_get_type
#define gtkMessageTypeGetType   gtk_message_type_get_type
#define gtkMetricTypeGetType   gtk_metric_type_get_type
#define gtkMovementStepGetType   gtk_movement_step_get_type
#define gtkOrientationGetType   gtk_orientation_get_type
#define gtkPackTypeGetType   gtk_pack_type_get_type
#define gtkPageOrientationGetType   gtk_page_orientation_get_type
#define gtkPrintOperationResultGetType   gtk_print_operation_result_get_type
#define gtkPrintPagesGetType   gtk_print_pages_get_type
#define gtkPrintQualityGetType   gtk_print_quality_get_type
#define gtkPrintStatusGetType   gtk_print_status_get_type
#define gtkPageSetGetType   gtk_page_set_get_type
#define gtkPathPriorityTypeGetType   gtk_path_priority_type_get_type
#define gtkPathTypeGetType   gtk_path_type_get_type
#define gtkPolicyTypeGetType   gtk_policy_type_get_type
#define gtkPositionTypeGetType   gtk_position_type_get_type
#define gtkPreviewTypeGetType   gtk_preview_type_get_type
#define gtkPrintDuplexGetType   gtk_print_duplex_get_type
#define gtkPrintErrorGetType   gtk_print_error_get_type
#define gtkRecentFilterFlagsGetType   gtk_recent_filter_flags_get_type
#define gtkPrintOperationActionGetType   gtk_print_operation_action_get_type
#define gtkPrivateFlagsGetType   gtk_private_flags_get_type
#define gtkProgressBarOrientationGetType   gtk_progress_bar_orientation_get_type
#define gtkProgressBarStyleGetType   gtk_progress_bar_style_get_type
#define gtkRcFlagsGetType   gtk_rc_flags_get_type
#define gtkRcTokenTypeGetType   gtk_rc_token_type_get_type
#define gtkRecentChooserErrorGetType   gtk_recent_chooser_error_get_type
#define gtkRecentManagerErrorGetType   gtk_recent_manager_error_get_type
#define gtkRecentSortTypeGetType   gtk_recent_sort_type_get_type
#define gtkUnitGetType   gtk_unit_get_type
#define gtkReliefStyleGetType   gtk_relief_style_get_type
#define gtkResizeModeGetType   gtk_resize_mode_get_type
#define gtkResponseTypeGetType   gtk_response_type_get_type
#define gtkScrollStepGetType   gtk_scroll_step_get_type
#define gtkScrollTypeGetType   gtk_scroll_type_get_type
#define gtkSelectionModeGetType   gtk_selection_mode_get_type
#define gtkShadowTypeGetType   gtk_shadow_type_get_type
#define gtkSideTypeGetType   gtk_side_type_get_type
#define gtkSignalRunTypeGetType   gtk_signal_run_type_get_type
#define gtkSizeGroupModeGetType   gtk_size_group_mode_get_type
#define gtkSortTypeGetType   gtk_sort_type_get_type
#define gtkSpinTypeGetType   gtk_spin_type_get_type
#define gtkStateTypeGetType   gtk_state_type_get_type
#define gtkSubmenuDirectionGetType   gtk_submenu_direction_get_type
#define gtkSubmenuPlacementGetType   gtk_submenu_placement_get_type
#define gtkTargetFlagsGetType   gtk_target_flags_get_type
#define gtkTextDirectionGetType   gtk_text_direction_get_type
#define gtkTextBufferTargetInfoGetType   gtk_text_buffer_target_info_get_type
#define gtkTextSearchFlagsGetType   gtk_text_search_flags_get_type
#define gtkTextWindowTypeGetType   gtk_text_window_type_get_type
#define gtkTreeModelFlagsGetType   gtk_tree_model_flags_get_type
#define gtkTreeViewModeGetType   gtk_tree_view_mode_get_type
#define gtkTreeViewGridLinesGetType   gtk_tree_view_grid_lines_get_type
#define gtkUpdateTypeGetType   gtk_update_type_get_type
#define gtkVisibilityGetType   gtk_visibility_get_type
#define gtkObjectFlagsGetType   gtk_object_flags_get_type
#define gtkAccelFlagsGetType   gtk_accel_flags_get_type
#define gtkIconSizeGetType   gtk_icon_size_get_type
#define gtkFileChooserErrorGetType   gtk_file_chooser_error_get_type
#define gtkIconThemeErrorGetType   gtk_icon_theme_error_get_type
#define gtkToolbarChildTypeGetType   gtk_toolbar_child_type_get_type
#define gtkToolbarSpaceStyleGetType   gtk_toolbar_space_style_get_type
#define gtkToolbarStyleGetType   gtk_toolbar_style_get_type
#define gtkArrowGetType   gtk_arrow_get_type
#define gtkAspectFrameGetType   gtk_aspect_frame_get_type
#define gtkBinGetType   gtk_bin_get_type
#define gtkBoxGetType   gtk_box_get_type
#define gtkBuildableGetType   gtk_buildable_get_type
#define gtkBuilderGetType   gtk_builder_get_type
#define gtkBuilderGetTypeFromName   gtk_builder_get_type_from_name
#define gtkButtonBoxGetType   gtk_button_box_get_type
#define gtkButtonGetType   gtk_button_get_type
#define gtkCalendarGetType   gtk_calendar_get_type
#define gtkCellEditableGetType   gtk_cell_editable_get_type
#define gtkCellLayoutGetType   gtk_cell_layout_get_type
#define gtkCellRendererGetType   gtk_cell_renderer_get_type
#define gtkCellRendererAccelGetType   gtk_cell_renderer_accel_get_type
#define gtkCellRendererComboGetType   gtk_cell_renderer_combo_get_type
#define gtkCellRendererPixbufGetType   gtk_cell_renderer_pixbuf_get_type
#define gtkCellRendererSpinGetType   gtk_cell_renderer_spin_get_type
#define gtkCellRendererProgressGetType   gtk_cell_renderer_progress_get_type
#define gtkCellRendererTextGetType   gtk_cell_renderer_text_get_type
#define gtkCellRendererToggleGetType   gtk_cell_renderer_toggle_get_type
#define gtkCellViewGetType   gtk_cell_view_get_type
#define gtkCheckButtonGetType   gtk_check_button_get_type
#define gtkCheckMenuItemGetType   gtk_check_menu_item_get_type
#define gtkClipboardGetType   gtk_clipboard_get_type
#define gtkClistGetType   gtk_clist_get_type
#define gtkColorButtonGetType   gtk_color_button_get_type
#define gtkColorSelectionGetType   gtk_color_selection_get_type
#define gtkColorSelectionDialogGetType   gtk_color_selection_dialog_get_type
#define gtkComboBoxGetType   gtk_combo_box_get_type
#define gtkComboBoxEntryGetType   gtk_combo_box_entry_get_type
#define gtkComboGetType   gtk_combo_get_type
#define gtkContainerGetType   gtk_container_get_type
#define gtkCtreeGetType   gtk_ctree_get_type
#define gtkCtreeNodeGetType   gtk_ctree_node_get_type
#define gtkCurveGetType   gtk_curve_get_type
#define gtkDialogGetType   gtk_dialog_get_type
#define gtkStatusIconGetType   gtk_status_icon_get_type
#define gtkBorderGetType   gtk_border_get_type
#define gtkStyleGetType   gtk_style_get_type
#define gtkDrawingAreaGetType   gtk_drawing_area_get_type
#define gtkEditableGetType   gtk_editable_get_type
#define gtkEntryGetType   gtk_entry_get_type
#define gtkEntryCompletionGetType   gtk_entry_completion_get_type
#define gtkEntryBufferGetType   gtk_entry_buffer_get_type
#define gtkEventBoxGetType   gtk_event_box_get_type
#define gtkExpanderGetType   gtk_expander_get_type
#define gtkFileChooserGetType   gtk_file_chooser_get_type
#define gtkFileChooserDialogGetType   gtk_file_chooser_dialog_get_type
#define gtkFileChooserWidgetGetType   gtk_file_chooser_widget_get_type
#define gtkFileChooserButtonGetType   gtk_file_chooser_button_get_type
#define gtkFileFilterGetType   gtk_file_filter_get_type
#define gtkFileSelectionGetType   gtk_file_selection_get_type
#define gtkFixedGetType   gtk_fixed_get_type
#define gtkFontButtonGetType   gtk_font_button_get_type
#define gtkFontSelectionDialogGetType   gtk_font_selection_dialog_get_type
#define gtkFontSelectionGetType   gtk_font_selection_get_type
#define gtkFrameGetType   gtk_frame_get_type
#define gtkGammaCurveGetType   gtk_gamma_curve_get_type
#define gtkHandleBoxGetType   gtk_handle_box_get_type
#define gtkHBoxGetType   gtk_hbox_get_type
#define gtkHButtonBoxGetType   gtk_hbutton_box_get_type
#define gtkHPanedGetType   gtk_hpaned_get_type
#define gtkHRulerGetType   gtk_hruler_get_type
#define gtkHScaleGetType   gtk_hscale_get_type
#define gtkHScrollbarGetType   gtk_hscrollbar_get_type
#define gtkHSeparatorGetType   gtk_hseparator_get_type
#define gtkHSVGetType   gtk_hsv_get_type
#define gtkIconFactoryGetType   gtk_icon_factory_get_type
#define gtkIconSetGetType   gtk_icon_set_get_type
#define gtkIconSourceGetType   gtk_icon_source_get_type
#define gtkIconInfoGetType   gtk_icon_info_get_type
#define gtkIconThemeGetType   gtk_icon_theme_get_type
#define gtkIconViewGetType   gtk_icon_view_get_type
#define gtkImageGetType   gtk_image_get_type
#define gtkImageMenuItemGetType   gtk_image_menu_item_get_type
#define gtkImContextGetType   gtk_im_context_get_type
#define gtkImContextSimpleGetType   gtk_im_context_simple_get_type
#define gtkImMulticontextGetType   gtk_im_multicontext_get_type
#define gtkInvisibleGetType   gtk_invisible_get_type
#define gtkItemGetType   gtk_item_get_type
#define gtkItemFactoryGetType   gtk_item_factory_get_type
#define gtkLabelGetType   gtk_label_get_type
#define gtkLayoutGetType   gtk_layout_get_type
#define gtkLinkButtonGetType   gtk_link_button_get_type
#define gtkListGetType   gtk_list_get_type
#define gtkListItemGetType   gtk_list_item_get_type
#define gtkListStoreGetType   gtk_list_store_get_type
#define gtkInputDialogGetType   gtk_input_dialog_get_type
#define gtkMenuBarGetType   gtk_menu_bar_get_type
#define gtkMenuGetType   gtk_menu_get_type
#define gtkMenuItemGetType   gtk_menu_item_get_type
#define gtkMenuShellGetType   gtk_menu_shell_get_type
#define gtkMenuToolButtonGetType   gtk_menu_tool_button_get_type
#define gtkMessageDialogGetType   gtk_message_dialog_get_type
#define gtkMiscGetType   gtk_misc_get_type
#define gtkMountOperationGetType   gtk_mount_operation_get_type
#define gtkNotebookGetType   gtk_notebook_get_type
#define gtkObjectGetType   gtk_object_get_type
#define gtkOrientableGetType   gtk_orientable_get_type
#define gtkOldEditableGetType   gtk_old_editable_get_type
#define gtkOptionMenuGetType   gtk_option_menu_get_type
#define gtkPanedGetType   gtk_paned_get_type
#define gtkPathBarGetType   gtk_path_bar_get_type
#define gtkPixmapGetType   gtk_pixmap_get_type
#define gtkPlugGetType   gtk_plug_get_type
#define gtkPreviewGetType   gtk_preview_get_type
#define gtkPageSetupGetType   gtk_page_setup_get_type
#define gtkPageSetupUnixDialogGetType   gtk_page_setup_unix_dialog_get_type
#define gtkPaperSizeGetType   gtk_paper_size_get_type
#define gtkCustomPaperUnixDialogGetType   gtk_custom_paper_unix_dialog_get_type
#define gtkPrintBackendGetType   gtk_print_backend_get_type
#define gtkPrintContextGetType   gtk_print_context_get_type
#define gtkPrinterGetType   gtk_printer_get_type
#define gtkPrintCapabilitiesGetType   gtk_print_capabilities_get_type
#define gtkPrinterOptionGetType   gtk_printer_option_get_type
#define gtkPrinterOptionSetGetType   gtk_printer_option_set_get_type
#define gtkPrinterOptionWidgetGetType   gtk_printer_option_widget_get_type
#define gtkPrintJobGetType   gtk_print_job_get_type
#define gtkPrintOperationGetType   gtk_print_operation_get_type
#define gtkPrintOperationPreviewGetType   gtk_print_operation_preview_get_type
#define gtkPrintSettingsGetType   gtk_print_settings_get_type
#define gtkPrintUnixDialogGetType   gtk_print_unix_dialog_get_type
#define gtkProgressBarGetType   gtk_progress_bar_get_type
#define gtkProgressGetType   gtk_progress_get_type
#define gtkRadioActionGetType   gtk_radio_action_get_type
#define gtkRadioButtonGetType   gtk_radio_button_get_type
#define gtkRadioMenuItemGetType   gtk_radio_menu_item_get_type
#define gtkRadioToolButtonGetType   gtk_radio_tool_button_get_type
#define gtkRangeGetType   gtk_range_get_type
#define gtkRcStyleGetType   gtk_rc_style_get_type
#define gtkRecentActionGetType   gtk_recent_action_get_type
#define gtkRecentChooserGetType   gtk_recent_chooser_get_type
#define gtkRecentChooserDialogGetType   gtk_recent_chooser_dialog_get_type
#define gtkRecentChooserMenuGetType   gtk_recent_chooser_menu_get_type
#define gtkRecentChooserWidgetGetType   gtk_recent_chooser_widget_get_type
#define gtkRecentFilterGetType   gtk_recent_filter_get_type
#define gtkRecentManagerGetType   gtk_recent_manager_get_type
#define gtkRecentInfoGetType   gtk_recent_info_get_type
#define gtkRulerGetType   gtk_ruler_get_type
#define gtkScaleGetType   gtk_scale_get_type
#define gtkScaleButtonGetType   gtk_scale_button_get_type
#define gtkScrollbarGetType   gtk_scrollbar_get_type
#define gtkScrolledWindowGetType   gtk_scrolled_window_get_type
#define gtkSelectionDataGetType   gtk_selection_data_get_type
#define gtkTargetListGetType   gtk_target_list_get_type
#define gtkSeparatorGetType   gtk_separator_get_type
#define gtkSeparatorMenuItemGetType   gtk_separator_menu_item_get_type
#define gtkSeparatorToolItemGetType   gtk_separator_tool_item_get_type
#define gtkSettingsGetType   gtk_settings_get_type
#define gtkSizeGroupGetType   gtk_size_group_get_type
#define gtkSocketGetType   gtk_socket_get_type
#define gtkSpinButtonGetType   gtk_spin_button_get_type
#define gtkStatusbarGetType   gtk_statusbar_get_type
#define gtkTableGetType   gtk_table_get_type
#define gtkTearoffMenuItemGetType   gtk_tearoff_menu_item_get_type
#define gtkTextAttributesGetType   gtk_text_attributes_get_type
#define gtkTextTagGetType   gtk_text_tag_get_type
#define gtkTextTagTableGetType   gtk_text_tag_table_get_type
#define gtkTextGetType   gtk_text_get_type
#define gtkTextBufferGetType   gtk_text_buffer_get_type
#define gtkTextIterGetType   gtk_text_iter_get_type
#define gtkTextLayoutGetType   gtk_text_layout_get_type
#define gtkTextChildAnchorGetType   gtk_text_child_anchor_get_type
#define gtkTextMarkGetType   gtk_text_mark_get_type
#define gtkTextViewGetType   gtk_text_view_get_type
#define gtkThemeEngineGetType   gtk_theme_engine_get_type
#define gtkTipsQueryGetType   gtk_tips_query_get_type
#define gtkToggleActionGetType   gtk_toggle_action_get_type
#define gtkToggleButtonGetType   gtk_toggle_button_get_type
#define gtkToggleToolButtonGetType   gtk_toggle_tool_button_get_type
#define gtkToolbarGetType   gtk_toolbar_get_type
#define gtkToolShellGetType   gtk_tool_shell_get_type
#define gtkToolButtonGetType   gtk_tool_button_get_type
#define gtkToolItemGetType   gtk_tool_item_get_type
#define gtkTooltipGetType   gtk_tooltip_get_type
#define gtkTooltipsGetType   gtk_tooltips_get_type
#define gtkTrayIconGetType   gtk_tray_icon_get_type
#define gtkTreeGetType   gtk_tree_get_type
#define gtkTreeDragDestGetType   gtk_tree_drag_dest_get_type
#define gtkTreeDragSourceGetType   gtk_tree_drag_source_get_type
#define gtkTreeItemGetType   gtk_tree_item_get_type
#define gtkTreeIterGetType   gtk_tree_iter_get_type
#define gtkTreeModelGetType   gtk_tree_model_get_type
#define gtkTreePathGetType   gtk_tree_path_get_type
#define gtkTreeRowReferenceGetType   gtk_tree_row_reference_get_type
#define gtkTreeModelSortGetType   gtk_tree_model_sort_get_type
#define gtkTreeModelFilterGetType   gtk_tree_model_filter_get_type
#define gtkTreeSelectionGetType   gtk_tree_selection_get_type
#define gtkTreeSortableGetType   gtk_tree_sortable_get_type
#define gtkTreeStoreGetType   gtk_tree_store_get_type
#define gtkTreeViewColumnGetType   gtk_tree_view_column_get_type
#define gtkTreeViewGetType   gtk_tree_view_get_type
#define gtkIdentifierGetType   gtk_identifier_get_type
#define gtkUiManagerGetType   gtk_ui_manager_get_type
#define gtkVBoxGetType   gtk_vbox_get_type
#define gtkVButtonBoxGetType   gtk_vbutton_box_get_type
#define gtkViewportGetType   gtk_viewport_get_type
#define gtkVolumeButtonGetType   gtk_volume_button_get_type
#define gtkVPanedGetType   gtk_vpaned_get_type
#define gtkVRulerGetType   gtk_vruler_get_type
#define gtkVScaleGetType   gtk_vscale_get_type
#define gtkVScrollbarGetType   gtk_vscrollbar_get_type
#define gtkVSeparatorGetType   gtk_vseparator_get_type
#define gtkRequisitionGetType   gtk_requisition_get_type
#define gtkWidgetGetType   gtk_widget_get_type
#define gtkWindowGetType   gtk_window_get_type
#define gtkWindowGetTypeHint   gtk_window_get_type_hint
#define gtkWindowGroupGetType   gtk_window_group_get_type
#define gtkWin32EmbedWidgetGetType   gtk_win32_embed_widget_get_type
#define gtkInfoBarGetType   gtk_info_bar_get_type

// AG 28/01/13
/* This is an awful hack to preload the Gtk widget types that we need. This is
   necessary because GtkBuilder will fail to instantiate types whose RTTI
   hasn't been loaded yet, and the types are only loaded lazily when they're
   needed. Catch 22. This is clearly a bug in GtkBuilder. The Glade builder
   does this properly (apparently using a similar hack), so this function
   doesn't have to be called there. */

static void builder_load_types()
{
	static int init = 0;

	if ( !init )
	{
		/* Make sure that we actually use these values in some way, so that gcc
		   doesn't optimize them away. */
		init = init |
			   gtkAccelLabelGetType() |
			   gtkLabelGetType() |
			   gtkProgressBarGetType() |
			   gtkStatusbarGetType() |
			   gtkStatusIconGetType() |
			   gtkButtonGetType() |
			   gtkCheckButtonGetType() |
			   gtkRadioButtonGetType() |
			   gtkToggleButtonGetType() |
			   gtkLinkButtonGetType() |
			   gtkScaleButtonGetType() |
			   gtkVolumeButtonGetType() |
			   gtkEntryGetType() |
			   gtkHScaleGetType() |
			   gtkVScaleGetType() |
			   gtkSpinButtonGetType() |
			   gtkEditableGetType() |
			   gtkTextViewGetType() |
			   gtkTreeViewGetType() |
			   gtkCellViewGetType() |
			   gtkIconViewGetType() |
			   gtkComboBoxGetType() |
			   gtkComboBoxEntryGetType() |
			   gtkMenuGetType() |
			   gtkMenuBarGetType() |
			   gtkMenuItemGetType() |
			   gtkToolbarGetType() |
			   gtkColorButtonGetType() |
			   gtkColorSelectionDialogGetType() |
			   gtkColorSelectionGetType() |
			   gtkHSVGetType() |
			   gtkFileChooserGetType() |
			   gtkFileChooserButtonGetType() |
			   gtkFileChooserDialogGetType() |
			   gtkFileChooserWidgetGetType() |
			   gtkFontButtonGetType() |
			   gtkFontSelectionDialogGetType() |
			   gtkAlignmentGetType() |
			   gtkHBoxGetType() |
			   gtkVBoxGetType() |
			   gtkHButtonBoxGetType() |
			   gtkVButtonBoxGetType() |
			   gtkFixedGetType() |
			   gtkHPanedGetType() |
			   gtkVPanedGetType() |
			   gtkLayoutGetType() |
			   gtkNotebookGetType() |
			   gtkTableGetType() |
			   gtkExpanderGetType() |
			   gtkFrameGetType() |
			   gtkHSeparatorGetType() |
			   gtkVSeparatorGetType() |
			   gtkHScrollbarGetType() |
			   gtkVScrollbarGetType() |
			   gtkScrolledWindowGetType() |
			   gtkPrintUnixDialogGetType() |
			   gtkPageSetupUnixDialogGetType() |
			   gtkArrowGetType() |
			   gtkCalendarGetType() |
			   gtkDrawingAreaGetType() |
			   gtkEventBoxGetType() |
			   gtkHandleBoxGetType() |
			   gtkWindowGetType() |
			   gtkToolButtonGetType() |
			   gtkImageGetType() |
			   gtkRecentChooserWidgetGetType() |
			   gtkAboutDialogGetType() |
			   gtkDialogGetType() |
			   gtkInputDialogGetType() |
			   gtkMessageDialogGetType() |
			   gtkRecentChooserDialogGetType() |
			   gtkAssistantGetType() |
			   1;
	}
}

/**
/brief  Load a builder file and create user interface.
        Register widget, and create variables holding names based upon id given in the builder file.
/author William J Giddings
/date   22-Nov-09
/todo   Obtain widget tree and then register widgets with gnocl.
**/
int gnoclBuilderCmd ( ClientData data, Tcl_Interp *interp, int objc, Tcl_Obj * const objv[] )
{

#ifdef DEBUG_BUILDER
	g_printf ( "gnoclBuilderCmd %d\n", objc );
	gint _i;

	for ( _i = 0; _i < objc; _i++ )
	{
		g_printf ( "\targ %d = %s\n", _i,  Tcl_GetString ( objv[_i] ) );
	}

#endif

	GtkBuilder *builder = NULL;
	GtkWidget *container;
	GtkWidget *objects;
	gchar *objname;
	gchar *cname;
	char *builder_ui;
	gchar str[128];
	GtkWidget *obj;
	GSList *objectList;
	GSList *widgetNames, *p;

// AG 28/01/13
	/* Make sure that we have loaded all the supported types beforehand,
	   otherwise GtkBuilder will fail to instantiate them. */
	builder_load_types();

	static const char *cmds[] = { "file", "buffer", NULL };
	enum optIdx { FileIdx, BufferIdx };
	int idx;


// objc must be either 3 or 7

	if ( ( objc != 3 ) && ( objc != 7 ) )
	{
		Tcl_SetResult ( interp, "GNOCL ERROR: Wrong number of arguments.\n", TCL_STATIC );
		return TCL_ERROR;
	}

	if ( Tcl_GetIndexFromObj ( interp, objv[1], cmds, "command", TCL_EXACT, &idx ) != TCL_OK )
	{
		Tcl_WrongNumArgs ( interp, 1, objv, "GNOCL ERROR: --- no message available ---\n" );
		return TCL_ERROR;
	}

	switch ( idx )
	{
		case FileIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( ".....load from file %s\n", Tcl_GetString ( objv[2] ) );
#endif

				sprintf ( str, "%s", Tcl_GetString ( objv[2] ) );
				//GtkWidget  *window1 = NULL;
				builder = gtk_builder_new ();
				gtk_builder_add_from_file ( builder, Tcl_GetString ( objv[2] ), NULL );
			}
			break;
		case BufferIdx:
			{

#ifdef DEBUG_BUILDER
				printf ( ".....load from buffer %s\n", Tcl_GetString ( objv[3] ) );
#endif

				switch ( objc )
				{
					case 3:
						{
#ifdef DEBUG_BUILDER
							printf ( "the whole buffer %s\n", Tcl_GetString ( objv[2] ) );
#endif
							//sprintf ( str, "%s", Tcl_GetString ( objv[2] ) );

							builder_ui = Tcl_GetString ( objv[2] );
							builder = gtk_builder_new ();

							gtk_builder_add_from_string ( builder, builder_ui, strlen ( builder_ui ) * sizeof ( gchar ), NULL );
							goto SwitchIdxEnd;
						}
						break;
					case 7:
						{

							/* is there some way of putting all this into a function in order to eliminate the goto?

							                            /* then must load a partial buffer into a container */
							if (  strcmp ( Tcl_GetString ( objv[3] ) , "-objects"  ) == 0
									&& strcmp ( Tcl_GetString ( objv[5] ) , "-container" ) == 0 )
							{

								objname = Tcl_GetString ( objv[4] );
								cname = Tcl_GetString ( objv[6] );
							}

							else if (  strcmp ( Tcl_GetString ( objv[5] ) , "-objects"  ) == 0
									   && strcmp ( Tcl_GetString ( objv[3] ) , "-container" ) == 0 )
							{
								objname = Tcl_GetString ( objv[6] );
								cname = Tcl_GetString ( objv[4] );
							}

						}
						break;
				} /* end switch(objc) */

				container = gnoclGetWidgetFromName ( cname, interp );

				if  ( !GTK_IS_CONTAINER ( container ) )
				{

					sprintf ( str, "GNOCL ERROR: Widget \"%s\" specfied by option -container is not a valid container.\n", cname );
					Tcl_SetResult ( interp, str, TCL_STATIC );
					return TCL_ERROR;
				}


				/* values obtained, error checking completed, now to load the buffer */
#ifdef DEBUG_BUILDER
				printf ( "Build -step 1\n" );
#endif
				builder_ui = Tcl_GetString ( objv[2] );
#ifdef DEBUG_BUILDER
				printf ( "Build -step 2\n" );
#endif                builder = gtk_builder_new();
#ifdef DEBUG_BUILDER
				printf ( "Build -step 3\n" );
				printf ( "builder_ui = %s \nobjname = %s\n", builder_ui, objname );
#endif
				gtk_builder_add_objects_from_string ( builder, builder_ui,
													  strlen ( builder_ui ) * sizeof ( gchar ),
													  g_strsplit_set ( objname, " ", -1 ), NULL );
#ifdef DEBUG_BUILDER
				printf ( "Build -step 4\n" );
#endif

				objects = gtk_builder_get_object ( builder, objname ) ;

#ifdef DEBUG_BUILDER
				printf ( "Build -step 5\n" );
				printf ( "Build -step 5a do we have a box conntainer?\n" );
#endif

				if ( !GTK_IS_BOX ( container ) )
				{

					if ( GTK_IS_WIDGET ( objects ) )
					{
						gtk_container_add ( GTK_CONTAINER ( container ), objects );
					}

					else
					{
						Tcl_SetResult ( interp, "GNOCL ERROR: The specified container can only hold one child widget.\n", TCL_STATIC );
						return TCL_ERROR;
					}

				}

				else
				{
#ifdef DEBUG_BUILDER
					printf ( "I am a box! Yippie, lets pack everything in then...!\n" );
#endif

					/* modify this section to pack all the elements of the array */
					int x = 0;
					char *str1;

					str1 = strtok ( objname, " " );
					//str1 =  strtok( g_strsplit_set ( objname, " ", -1 ), " " );

					if ( str1 == NULL )
					{
						/* just one object */
#ifdef DEBUG_BUILDER
						printf ( "just one object\n" );
#endif
						gtk_box_pack_end ( GTK_CONTAINER ( container ), GTK_WIDGET ( objects ), 1, 1, 0 );
						break;
					}

#ifdef DEBUG_BUILDER
					printf ( "pack first object object\n" );
#endif
					objects = gtk_builder_get_object ( builder, str1 ) ;
					gtk_box_pack_end ( GTK_CONTAINER ( container ), GTK_WIDGET ( objects ), 1, 1, 0 );

					/* otherwise, loop until finishied */
					while ( 1 )
					{
						/* extract next item from string sequence */
						str1 = strtok ( NULL, " " );


						/* check if there is nothing else to extract */
						if ( str1 == NULL )
						{

							break;
						}

						/* print string after tokenized */
#ifdef DEBUG_BUILDER
						printf ( "\n\n>>>>>%i: %s\n\n", x, str1 );
#endif

						objects = gtk_builder_get_object ( builder, str1 ) ;

#ifdef DEBUG_BUILDER
						printf ( "Build -step 5\n" );
#endif
						// gtk_box_pack_end (GtkBox *box, GtkWidget *child, gboolean expand, gboolean fill, guint padding);
						gtk_box_pack_start ( GTK_CONTAINER ( container ), GTK_WIDGET ( objects ), 1, 1, 0 );
						//gtk_container_add ( GTK_CONTAINER ( container ), GTK_WIDGET ( objects ) );

						x++;

					}

#ifdef DEBUG_BUILDER
					printf ( "Build -step 6\n" );
#endif
				} /* end else */


			} /* end case BufferIdx: */
	} /* end switch(idx) */

SwitchIdxEnd:

	/* the convenience function left out of the basic set of library functions */
	widgetNames = getBuilderObjectNames ( builder );
#ifdef DEBUG_BUILDER
	display_list ( widgetNames );
#endif

	GtkWidget *window;
	GtkWidget *widget;


	/*  build up a list of names */
	Tcl_Obj *resList = Tcl_NewListObj ( 0, NULL );

	/* ignore gnomeApp widgets, these are handled in their own modules */
	for ( p = widgetNames; p != NULL; p = p->next )
	{
#ifdef DEBUG_BUILDER
		printf ( "Now sorting out handler function for %s\n", p->data );
#endif
		/* obtain pointer to widget based upon list of names */
		widget = gtk_builder_get_object ( builder, p->data );

		/* obtain a new gnocl name */
		const char *name = gnoclGetAutoWidgetId();

		gchar *type =  G_OBJECT_TYPE_NAME ( widget );

#ifdef DEBUG_BUILDER == builder
		printf ( "The gnocl name created is %s, type = %s\n", name, type );
#endif
		/* determine how to handle the new widget */
		gint idx = compare ( type, GtkTypes );
#ifdef DEBUG_BUILDER == builder
		printf ( "The gnocl name created is %s, type = %s, idx = %d\n", name, type, idx );
#endif
		/* now, effectively a multi-widget gnoclRegisterWidget function */
		setGtkWidgetFunc ( interp, widget, idx , resList, p->data, name );

		if ( 0 )
		{
			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkWindow" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkWindow\n" );
#endif
				window = GTK_WIDGET (  p->data );

			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );

			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkAboutDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkAboutDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkColorSelectionDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkColorSelectionDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkFileChooserDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkFileChooserDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkFontSelectionDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkFontSelectionDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkInputDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkInputDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkMessageDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkMessageDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkRecentChooserDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkRecentChooserDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkAssistant" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkAssistant\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

		}

	}

	/* create a specific command for each of the builder files loaded */
	Tcl_CreateObjCommand ( interp, Tcl_GetString ( objv[2] ), builderFunc, builder, NULL );

	/* return the list of created widgets */
	Tcl_SetObjResult ( interp, resList );
	return TCL_OK;

}


/* glade equivalents */

/**
\brief
\author William J Giddings
\date   23/11/09
\note
*/
static int gladeFunc ( ClientData data, Tcl_Interp *interp, int objc, Tcl_Obj * const objv[] )
{
#ifdef DEBUG_BUILDER
	g_print ( " gladeFunc, this will provide a command to access items(?) resulting from the loading of this tree!\n" );
#endif
	return TCL_OK;
}


/**
\brief  Dummy function for assigning to newly created gnocl widget within this module
\author William J Giddings
\date   23/11/09
\note
*/
static int gladeWidgetFunc ( ClientData data, Tcl_Interp *interp, int objc, Tcl_Obj * const objv[] )
{
#ifdef DEBUG_BUILDER
	g_print ( " gladeWidgetFunc, this is just a stub!\n" );
#endif
	return TCL_OK;
}





/**
/brief  Load a Glade file and create user interface.
        Register widget, and create variables holding names based upon id given in the glade file.
/author William J Giddings
/date   22-Nov-09
/todo   Obtain widget tree and then register widgets with gnocl.
**/
int gnoclGladeCmd (
	ClientData data,
	Tcl_Interp *interp,
	int objc,
	Tcl_Obj * const objv[] )
{
	static const char *cmds[] = { "file", "buffer", NULL };
	enum optIdx { FileIdx, BufferIdx };
	int idx;

	GladeXML *xml;

	if ( objc != 3 )
	{
		Tcl_WrongNumArgs ( interp, 1, objv, "GNOCL ERROR: option widgetid " );
		return TCL_ERROR;
	}

	if ( Tcl_GetIndexFromObj ( interp, objv[1], cmds, "command", TCL_EXACT, &idx ) != TCL_OK )
	{
		return TCL_ERROR;
	}

	switch ( idx )
	{
		case FileIdx:
			{
#ifdef DEBUG_BUILDER
				printf ( ".....load from file\n" );
#endif
				xml = glade_xml_new ( Tcl_GetString ( objv[2] ), NULL, NULL );
			}
			break;
		case BufferIdx:
			{
				char *glade_ui = Tcl_GetString ( objv[2] );
				xml = glade_xml_new_from_buffer ( glade_ui, strlen ( glade_ui ) * sizeof ( gchar ), NULL, NULL );
#ifdef DEBUG_BUILDER
				printf ( ".....load from buffer\n" );
#endif
			}
			break;
	}

	GtkWidget *window;
	GtkWidget *widget;

	GList *widgetList, *p;
	gchar str[50];

	/* process remaining widgets in the gui description */
	widgetList = glade_xml_get_widget_prefix ( xml, "" );

	/*  build up a list of names */
	Tcl_Obj *resList = Tcl_NewListObj ( 0, NULL );

	/* ignore gnomeApp widgets, these are handled in their own modules */
	for ( p = widgetList; p != NULL; p = p->next )
	{

		widget = GTK_WIDGET (  p->data );

		const char *name = gnoclGetAutoWidgetId();

		/* name and type, add to list */
		gchar *gladeName = glade_get_widget_name ( widget );

		gchar *type =  G_OBJECT_TYPE_NAME ( widget );

		/* determine how to handle the new widget */
		gint idx = compare ( type, GtkTypes );

		/* now, effectively a multi-widget gnoclRegisterWidget function */
		setGtkWidgetFunc ( interp, widget, idx , resList, gladeName, name );

		if ( 0 )
		{
			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkWindow" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkWindow\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkAboutDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkAboutDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkColorSelectionDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkColorSelectionDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkFileChooserDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkFileChooserDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkFontSelectionDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkFontSelectionDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkInputDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkInputDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkMessageDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkMessageDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkRecentChooserDialog" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkRecentChooserDialog\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

			/* search out the toplevel window for this file */
			if ( strcmp ( type, "GtkAssistant" ) == 0 )
			{
#ifdef DEBUG_BUILDER
				g_print ( "I am the GtkAssistant\n" );
#endif
				window = GTK_WIDGET (  p->data );
			}

		}

	}

	/* create a specific command for each of the glade files loaded */
	Tcl_CreateObjCommand ( interp, Tcl_GetString ( objv[2] ), gladeFunc, xml, NULL );

	/* return the list of created widgets */
	Tcl_SetObjResult ( interp, resList );
	return TCL_OK;

}

